@@ -36,6 +36,8 @@ gsl_matrix * create_vandermonde_matrix(size_t size);
 gsl_matrix * create_moler_matrix(size_t size);
 gsl_matrix * create_row_matrix(size_t size1, size_t size2);
 gsl_matrix * create_2x2_matrix(double a11, double a12, double a21, double a22);
+gsl_matrix * create_diagonal_matrix(double a[], size_t size);
+
 int test_matmult(void);
 int test_matmult_mod(void);
 int test_LU_solve_dim(const gsl_matrix * m, const double * actual, double eps);
@@ -44,12 +46,16 @@ int test_LUc_solve_dim(const gsl_matrix_complex * m, const double * actual, doub
 int test_LUc_solve(void);
 int test_QR_solve_dim(const gsl_matrix * m, const double * actual, double eps);
 int test_QR_solve(void);
+int test_QR_QRsolve_dim(const gsl_matrix * m, const double * actual, double eps);
+int test_QR_QRsolve(void);
 int test_QR_lssolve_dim(const gsl_matrix * m, const double * actual, double eps);
 int test_QR_lssolve(void);
 int test_QR_decomp_dim(const gsl_matrix * m, double eps);
 int test_QR_decomp(void);
 int test_QRPT_solve_dim(const gsl_matrix * m, const double * actual, double eps);
 int test_QRPT_solve(void);
+int test_QRPT_QRsolve_dim(const gsl_matrix * m, const double * actual, double eps);
+int test_QRPT_QRsolve(void);
 int test_QRPT_decomp_dim(const gsl_matrix * m, double eps);
 int test_QRPT_decomp(void);
 int test_QR_update_dim(const gsl_matrix * m, double eps);
@@ -58,19 +64,21 @@ int test_SV_solve_dim(const gsl_matrix * m, const double * actual, double eps);
 int test_SV_solve(void);
 int test_SV_decomp_dim(const gsl_matrix * m, double eps);
 int test_SV_decomp(void);
+int test_SV_decomp_mod_dim(const gsl_matrix * m, double eps);
+int test_SV_decomp_mod(void);
 int test_cholesky_solve_dim(const gsl_matrix * m, const double * actual, double eps);
 int test_cholesky_solve(void);
 int test_cholesky_decomp_dim(const gsl_matrix * m, double eps);
 int test_cholesky_decomp(void);
 int test_HH_solve_dim(const gsl_matrix * m, const double * actual, double eps);
 int test_HH_solve(void);
-int test_TD_solve_dim(size_t dim, double d, double od, const double * actual, double eps);
-int test_TD_solve(void);
+int test_TDS_solve_dim(size_t dim, double d, double od, const double * actual, double eps);
+int test_TDS_solve(void);
 int test_TDN_solve_dim(size_t dim, double d, double a, double b, const double * actual, double eps);
 int test_TDN_solve(void);
-int test_TD_cyc_solve_one(const size_t dim, const double * d, const double * od, const double * r,
+int test_TDS_cyc_solve_one(const size_t dim, const double * d, const double * od, const double * r,
                           const double * actual, double eps);
-int test_TD_cyc_solve(void);
+int test_TDS_cyc_solve(void);
 int test_TDN_cyc_solve_dim(size_t dim, double d, double a, double b, const double * actual, double eps);
 int test_TDN_cyc_solve(void);
 int test_bidiag_decomp_dim(const gsl_matrix * m, double eps);
@@ -79,10 +87,18 @@ int test_bidiag_decomp(void);
 int 
 check (double x, double actual, double eps)
 {
-  if (actual == 0)
-    return fabs(x) > eps;
+  if (x == actual)
+    {
+      return 0;
+    }
+  else if (actual == 0)
+    {
+      return fabs(x) > eps;
+    }
   else
-    return (fabs(x - actual)/fabs(actual)) > eps;
+    {
+      return (fabs(x - actual)/fabs(actual)) > eps;
+    }
 }
 
 gsl_matrix *
@@ -193,6 +209,21 @@ create_2x2_matrix(double a11, double a12, double a21, double a22)
   return m;
 }
 
+gsl_matrix *
+create_diagonal_matrix(double a[], size_t size)
+{
+  size_t i;
+  gsl_matrix * m = gsl_matrix_calloc(size, size);
+  for(i=0; i<size; i++) {
+      gsl_matrix_set(m, i, i, a[i]);
+  }
+
+  return m;
+}
+
+gsl_matrix * m11;
+gsl_matrix * m51;
+
 gsl_matrix * m35;
 gsl_matrix * m53;
 gsl_matrix * m97;
@@ -215,6 +246,9 @@ gsl_matrix * A44;
 
 gsl_matrix_complex * c7;
 
+gsl_matrix * inf5; double inf5_data[] = {1.0, 0.0, -3.0, 0.0, -5.0};
+gsl_matrix * nan5;
+
 double m53_lssolution[] = {52.5992295702070, -337.7263113752073, 
                            351.8823436427604};
 double hilb2_solution[] = {-8.0, 18.0} ;
@@ -466,7 +500,7 @@ int test_LU_solve(void)
   gsl_test(f, "  LU_solve hilbert(3)");
   s += f;
 
-  f = test_LU_solve_dim(hilb4, hilb4_solution, 1024.0 * GSL_DBL_EPSILON);
+  f = test_LU_solve_dim(hilb4, hilb4_solution, 2048.0 * GSL_DBL_EPSILON);
   gsl_test(f, "  LU_solve hilbert(4)");
   s += f;
 
@@ -634,6 +668,81 @@ int test_QR_solve(void)
 }
 
 
+int
+test_QR_QRsolve_dim(const gsl_matrix * m, const double * actual, double eps)
+{
+  int s = 0;
+  size_t i, dim = m->size1;
+
+  gsl_vector * rhs = gsl_vector_alloc(dim);
+  gsl_matrix * qr  = gsl_matrix_alloc(dim,dim);
+  gsl_matrix * q  = gsl_matrix_alloc(dim,dim);
+  gsl_matrix * r  = gsl_matrix_alloc(dim,dim);
+  gsl_vector * d = gsl_vector_alloc(dim);
+  gsl_vector * x = gsl_vector_alloc(dim);
+
+  gsl_matrix_memcpy(qr,m);
+  for(i=0; i<dim; i++) gsl_vector_set(rhs, i, i+1.0);
+  s += gsl_linalg_QR_decomp(qr, d);
+  s += gsl_linalg_QR_unpack(qr, d, q, r);
+  s += gsl_linalg_QR_QRsolve(q, r, rhs, x);
+  for(i=0; i<dim; i++) {
+    int foo = check(gsl_vector_get(x, i), actual[i], eps);
+    if(foo) {
+      printf("%3d[%d]: %22.18g   %22.18g\n", dim, i, gsl_vector_get(x, i), actual[i]);
+    }
+    s += foo;
+  }
+
+  gsl_vector_free(x);
+  gsl_vector_free(d);
+  gsl_matrix_free(qr);
+  gsl_vector_free(rhs);
+
+  return s;
+}
+
+int test_QR_QRsolve(void)
+{
+  int f;
+  int s = 0;
+
+  f = test_QR_QRsolve_dim(hilb2, hilb2_solution, 2 * 8.0 * GSL_DBL_EPSILON);
+  gsl_test(f, "  QR_QRsolve hilbert(2)");
+  s += f;
+
+  f = test_QR_QRsolve_dim(hilb3, hilb3_solution, 2 * 64.0 * GSL_DBL_EPSILON);
+  gsl_test(f, "  QR_QRsolve hilbert(3)");
+  s += f;
+
+  f = test_QR_QRsolve_dim(hilb4, hilb4_solution, 2 * 1024.0 * GSL_DBL_EPSILON);
+  gsl_test(f, "  QR_QRsolve hilbert(4)");
+  s += f;
+
+  f = test_QR_QRsolve_dim(hilb12, hilb12_solution, 0.5);
+  gsl_test(f, "  QR_QRsolve hilbert(12)");
+  s += f;
+
+  f = test_QR_QRsolve_dim(vander2, vander2_solution, 8.0 * GSL_DBL_EPSILON);
+  gsl_test(f, "  QR_QRsolve vander(2)");
+  s += f;
+
+  f = test_QR_QRsolve_dim(vander3, vander3_solution, 64.0 * GSL_DBL_EPSILON);
+  gsl_test(f, "  QR_QRsolve vander(3)");
+  s += f;
+
+  f = test_QR_QRsolve_dim(vander4, vander4_solution, 1024.0 * GSL_DBL_EPSILON);
+  gsl_test(f, "  QR_QRsolve vander(4)");
+  s += f;
+
+  f = test_QR_QRsolve_dim(vander12, vander12_solution, 0.05);
+  gsl_test(f, "  QR_QRsolve vander(12)");
+  s += f;
+
+  return s;
+}
+
+
 int
 test_QR_lssolve_dim(const gsl_matrix * m, const double * actual, double eps)
 {
@@ -896,6 +1005,83 @@ int test_QRPT_solve(void)
   return s;
 }
 
+int
+test_QRPT_QRsolve_dim(const gsl_matrix * m, const double * actual, double eps)
+{
+  int s = 0;
+  int signum;
+  size_t i, dim = m->size1;
+
+  gsl_permutation * perm = gsl_permutation_alloc(dim);
+  gsl_vector * rhs = gsl_vector_alloc(dim);
+  gsl_matrix * qr  = gsl_matrix_alloc(dim,dim);
+  gsl_matrix * q  = gsl_matrix_alloc(dim,dim);
+  gsl_matrix * r  = gsl_matrix_alloc(dim,dim);
+  gsl_vector * d = gsl_vector_alloc(dim);
+  gsl_vector * x = gsl_vector_alloc(dim);
+  gsl_vector * norm = gsl_vector_alloc(dim);
+
+  gsl_matrix_memcpy(qr,m);
+  for(i=0; i<dim; i++) gsl_vector_set(rhs, i, i+1.0);
+  s += gsl_linalg_QRPT_decomp2(qr, q, r, d, perm, &signum, norm);
+  s += gsl_linalg_QRPT_QRsolve(q, r, perm, rhs, x);
+  for(i=0; i<dim; i++) {
+    int foo = check(gsl_vector_get(x, i), actual[i], eps);
+    if(foo) {
+      printf("%3d[%d]: %22.18g   %22.18g\n", dim, i, gsl_vector_get(x, i), actual[i]);
+    }
+    s += foo;
+  }
+
+  gsl_vector_free(norm);
+  gsl_vector_free(x);
+  gsl_vector_free(d);
+  gsl_matrix_free(qr);
+  gsl_vector_free(rhs);
+  gsl_permutation_free(perm);
+
+  return s;
+}
+
+int test_QRPT_QRsolve(void)
+{
+  int f;
+  int s = 0;
+
+  f = test_QRPT_QRsolve_dim(hilb2, hilb2_solution, 2 * 8.0 * GSL_DBL_EPSILON);
+  gsl_test(f, "  QRPT_QRsolve hilbert(2)");
+  s += f;
+
+  f = test_QRPT_QRsolve_dim(hilb3, hilb3_solution, 2 * 64.0 * GSL_DBL_EPSILON);
+  gsl_test(f, "  QRPT_QRsolve hilbert(3)");
+  s += f;
+
+  f = test_QRPT_QRsolve_dim(hilb4, hilb4_solution, 2 * 2048.0 * GSL_DBL_EPSILON);
+  gsl_test(f, "  QRPT_QRsolve hilbert(4)");
+  s += f;
+
+  f = test_QRPT_QRsolve_dim(hilb12, hilb12_solution, 0.5);
+  gsl_test(f, "  QRPT_QRsolve hilbert(12)");
+  s += f;
+
+  f = test_QRPT_QRsolve_dim(vander2, vander2_solution, 8.0 * GSL_DBL_EPSILON);
+  gsl_test(f, "  QRPT_QRsolve vander(2)");
+  s += f;
+
+  f = test_QRPT_QRsolve_dim(vander3, vander3_solution, 64.0 * GSL_DBL_EPSILON);
+  gsl_test(f, "  QRPT_QRsolve vander(3)");
+  s += f;
+
+  f = test_QRPT_QRsolve_dim(vander4, vander4_solution, 1024.0 * GSL_DBL_EPSILON);
+  gsl_test(f, "  QRPT_QRsolve vander(4)");
+  s += f;
+
+  f = test_QRPT_QRsolve_dim(vander12, vander12_solution, 0.05);
+  gsl_test(f, "  QRPT_QRsolve vander(12)");
+  s += f;
+
+  return s;
+}
 
 int
 test_QRPT_decomp_dim(const gsl_matrix * m, double eps)
@@ -1227,7 +1413,6 @@ int test_SV_solve(void)
   return s;
 }
 
-
 int
 test_SV_decomp_dim(const gsl_matrix * m, double eps)
 {
@@ -1244,7 +1429,7 @@ test_SV_decomp_dim(const gsl_matrix * m, double eps)
 
   gsl_matrix_memcpy(v,m);
 
-  s += gsl_linalg_SV_decomp(v, q, d, w);
+  s += gsl_linalg_SV_decomp(v, q, d, w); 
 
   /* Check that singular values are non-negative and in non-decreasing
      order */
@@ -1255,6 +1440,11 @@ test_SV_decomp_dim(const gsl_matrix * m, double eps)
     {
       double di = gsl_vector_get (d, i);
 
+      if (gsl_isnan (di))
+        {
+          continue;  /* skip NaNs */
+        }
+
       if (di < 0) {
         s++;
         printf("singular value %d = %22.18g < 0\n", i, di);
@@ -1310,6 +1500,14 @@ int test_SV_decomp(void)
   int f;
   int s = 0;
 
+  f = test_SV_decomp_dim(m11, 2 * GSL_DBL_EPSILON);
+  gsl_test(f, "  SV_decomp m(1,1)");
+  s += f;
+
+  f = test_SV_decomp_dim(m51, 2 * 64.0 * GSL_DBL_EPSILON);
+  gsl_test(f, "  SV_decomp m(5,1)");
+  s += f;
+
   /* M<N not implemented yet */
 #if 0
   f = test_SV_decomp_dim(m35, 2 * 8.0 * GSL_DBL_EPSILON);
@@ -1368,6 +1566,15 @@ int test_SV_decomp(void)
   gsl_test(f, "  SV_decomp row12");
   s += f;
 
+  f = test_SV_decomp_dim(inf5, 1024 * GSL_DBL_EPSILON);
+  gsl_test(f, "  SV_decomp inf5");
+  s += f;
+
+  f = test_SV_decomp_dim(nan5, 1024 * GSL_DBL_EPSILON);
+  gsl_test(f, "  SV_decomp nan5");
+  s += f;
+
+
   {
     double i1, i2, i3, i4;
     double lower = -2, upper = 2;
@@ -1451,6 +1658,252 @@ int test_SV_decomp(void)
 }
 
 
+int
+test_SV_decomp_mod_dim(const gsl_matrix * m, double eps)
+{
+  int s = 0;
+  double di1;
+  size_t i,j, M = m->size1, N = m->size2;
+
+  gsl_matrix * v  = gsl_matrix_alloc(M,N);
+  gsl_matrix * a  = gsl_matrix_alloc(M,N);
+  gsl_matrix * q  = gsl_matrix_alloc(N,N);
+  gsl_matrix * x  = gsl_matrix_alloc(N,N);
+  gsl_matrix * dqt  = gsl_matrix_alloc(N,N);
+  gsl_vector * d  = gsl_vector_alloc(N);
+  gsl_vector * w  = gsl_vector_alloc(N);
+
+  gsl_matrix_memcpy(v,m);
+
+  s += gsl_linalg_SV_decomp_mod(v, x, q, d, w); 
+
+  /* Check that singular values are non-negative and in non-decreasing
+     order */
+  
+  di1 = 0.0;
+
+  for (i = 0; i < N; i++)
+    {
+      double di = gsl_vector_get (d, i);
+
+      if (gsl_isnan (di))
+        {
+          continue;  /* skip NaNs */
+        }
+
+      if (di < 0) {
+        s++;
+        printf("singular value %d = %22.18g < 0\n", i, di);
+      }
+
+      if(i > 0 && di > di1) {
+        s++;
+        printf("singular value %d = %22.18g vs previous %22.18g\n", i, di, di1);
+      }
+
+      di1 = di;
+    }      
+  
+  /* Scale dqt = D Q^T */
+  
+  for (i = 0; i < N ; i++)
+    {
+      double di = gsl_vector_get (d, i);
+
+      for (j = 0; j < N; j++)
+        {
+          double qji = gsl_matrix_get(q, j, i);
+          gsl_matrix_set (dqt, i, j, qji * di);
+        }
+    }
+            
+  /* compute a = v dqt */
+  gsl_blas_dgemm (CblasNoTrans, CblasNoTrans, 1.0, v, dqt, 0.0, a);
+
+  for(i=0; i<M; i++) {
+    for(j=0; j<N; j++) {
+      double aij = gsl_matrix_get(a, i, j);
+      double mij = gsl_matrix_get(m, i, j);
+      int foo = check(aij, mij, eps);
+      if(foo) {
+        printf("(%3d,%3d)[%d,%d]: %22.18g   %22.18g\n", M, N, i,j, aij, mij);
+      }
+      s += foo;
+    }
+  }
+  gsl_vector_free(w);
+  gsl_vector_free(d);
+  gsl_matrix_free(v);
+  gsl_matrix_free(a);
+  gsl_matrix_free(q);
+  gsl_matrix_free(dqt);
+
+  return s;
+}
+
+int test_SV_decomp_mod(void)
+{
+  int f;
+  int s = 0;
+
+  f = test_SV_decomp_mod_dim(m11, 2 * GSL_DBL_EPSILON);
+  gsl_test(f, "  SV_decomp_mod m(1,1)");
+  s += f;
+
+  f = test_SV_decomp_mod_dim(m51, 2 * 64.0 * GSL_DBL_EPSILON);
+  gsl_test(f, "  SV_decomp_mod m(5,1)");
+  s += f;
+
+  /* M<N not implemented yet */
+#if 0
+  f = test_SV_decomp_mod_dim(m35, 2 * 8.0 * GSL_DBL_EPSILON);
+  gsl_test(f, "  SV_decomp_mod m(3,5)");
+  s += f;
+#endif
+  f = test_SV_decomp_mod_dim(m53, 2 * 64.0 * GSL_DBL_EPSILON);
+  gsl_test(f, "  SV_decomp_mod m(5,3)");
+  s += f;
+
+  f = test_SV_decomp_mod_dim(moler10, 2 * 64.0 * GSL_DBL_EPSILON);
+  gsl_test(f, "  SV_decomp_mod moler(10)");
+  s += f;
+
+  f = test_SV_decomp_mod_dim(hilb2, 2 * 8.0 * GSL_DBL_EPSILON);
+  gsl_test(f, "  SV_decomp_mod hilbert(2)");
+  s += f;
+
+  f = test_SV_decomp_mod_dim(hilb3, 2 * 64.0 * GSL_DBL_EPSILON);
+  gsl_test(f, "  SV_decomp_mod hilbert(3)");
+  s += f;
+
+  f = test_SV_decomp_mod_dim(hilb4, 2 * 1024.0 * GSL_DBL_EPSILON);
+  gsl_test(f, "  SV_decomp_mod hilbert(4)");
+  s += f;
+
+  f = test_SV_decomp_mod_dim(hilb12, 2 * 1024.0 * GSL_DBL_EPSILON);
+  gsl_test(f, "  SV_decomp_mod hilbert(12)");
+  s += f;
+
+  f = test_SV_decomp_mod_dim(vander2, 8.0 * GSL_DBL_EPSILON);
+  gsl_test(f, "  SV_decomp_mod vander(2)");
+  s += f;
+
+  f = test_SV_decomp_mod_dim(vander3, 64.0 * GSL_DBL_EPSILON);
+  gsl_test(f, "  SV_decomp_mod vander(3)");
+  s += f;
+
+  f = test_SV_decomp_mod_dim(vander4, 1024.0 * GSL_DBL_EPSILON);
+  gsl_test(f, "  SV_decomp_mod vander(4)");
+  s += f;
+
+  f = test_SV_decomp_mod_dim(vander12, 1e-4);
+  gsl_test(f, "  SV_decomp_mod vander(12)");
+  s += f;
+
+  f = test_SV_decomp_mod_dim(row3, 10 * GSL_DBL_EPSILON);
+  gsl_test(f, "  SV_decomp_mod row3");
+  s += f;
+
+  f = test_SV_decomp_mod_dim(row5, 128 * GSL_DBL_EPSILON);
+  gsl_test(f, "  SV_decomp_mod row5");
+  s += f;
+
+  f = test_SV_decomp_mod_dim(row12, 1024 * GSL_DBL_EPSILON);
+  gsl_test(f, "  SV_decomp_mod row12");
+  s += f;
+
+  f = test_SV_decomp_mod_dim(inf5, 1024 * GSL_DBL_EPSILON);
+  gsl_test(f, "  SV_decomp_mod inf5");
+  s += f;
+
+  f = test_SV_decomp_mod_dim(nan5, 1024 * GSL_DBL_EPSILON);
+  gsl_test(f, "  SV_decomp_mod nan5");
+  s += f;
+
+
+  {
+    double i1, i2, i3, i4;
+    double lower = -2, upper = 2;
+
+    for (i1 = lower; i1 <= upper; i1++)
+      {
+        for (i2 = lower; i2 <= upper; i2++)
+          {
+            for (i3 = lower; i3 <= upper; i3++)
+              {
+                for (i4 = lower; i4 <= upper; i4++)
+                  {
+                    gsl_matrix_set (A22, 0,0, i1);
+                    gsl_matrix_set (A22, 0,1, i2);
+                    gsl_matrix_set (A22, 1,0, i3);
+                    gsl_matrix_set (A22, 1,1, i4);
+                    
+                    f = test_SV_decomp_mod_dim(A22, 16 * GSL_DBL_EPSILON);
+                    gsl_test(f, "  SV_decomp_mod (2x2) A=[%g, %g; %g, %g]", i1,i2,i3,i4);
+                    s += f;
+                  }
+              }
+          }
+      }
+  }
+
+  {
+    int i;
+    double carry = 0, lower = 0, upper = 1;
+    double *a = A33->data;
+
+    for (i=0; i<9; i++) {
+      a[i] = lower;
+    }
+    
+    while (carry == 0.0) {
+      f = test_SV_decomp_mod_dim(A33, 64 * GSL_DBL_EPSILON);
+      gsl_test(f, "  SV_decomp_mod (3x3) A=[ %g, %g, %g; %g, %g, %g; %g, %g, %g]",
+               a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8]);
+      
+      /* increment */
+      carry=1.0;
+      for (i=9; i>0 && i--;) 
+        {
+          double v=a[i]+carry;
+          carry = (v>upper) ? 1.0 : 0.0;
+          a[i] = (v>upper) ? lower : v;
+        }
+    }
+  }
+
+#ifdef TEST_SVD_4X4
+  {
+    int i;
+    double carry = 0, lower = 0, upper = 1;
+    double *a = A44->data;
+
+    for (i=0; i<16; i++) {
+      a[i] = lower;
+    }
+    
+    while (carry == 0.0) {
+      f = test_SV_decomp_mod_dim(A44, 64 * GSL_DBL_EPSILON);
+      gsl_test(f, "  SV_decomp_mod (4x4) A=[ %g, %g, %g, %g; %g, %g, %g, %g; %g, %g, %g, %g; %g, %g, %g, %g]",
+               a[0], a[1], a[2], a[3], a[4], a[5], a[6], a[7], a[8], a[9],
+               a[10], a[11], a[12], a[13], a[14], a[15]);
+      
+      /* increment */
+      carry=1.0;
+      for (i=16; i>0 && i--;) 
+        {
+          double v=a[i]+carry;
+          carry = (v>upper) ? 1.0 : 0.0;
+          a[i] = (v>upper) ? lower : v;
+        }
+    }
+  }
+#endif
+
+  return s;
+}
+
+
 int
 test_cholesky_solve_dim(const gsl_matrix * m, const double * actual, double eps)
 {
@@ -1648,7 +2101,7 @@ int test_HH_solve(void)
 
 
 int
-test_TD_solve_dim(size_t dim, double d, double od, const double * actual, double eps)
+test_TDS_solve_dim(size_t dim, double d, double od, const double * actual, double eps)
 {
   int s = 0;
   size_t i;
@@ -1687,38 +2140,37 @@ test_TD_solve_dim(size_t dim, double d, double od, const double * actual, double
 }
 
 
-int test_TD_solve(void)
+int test_TDS_solve(void)
 {
   int f;
   int s = 0;
-  double actual[16];
 
-  actual[0] =  0.0;
-  actual[1] =  2.0;
-  f = test_TD_solve_dim(2, 1.0, 0.5, actual, 8.0 * GSL_DBL_EPSILON);
-  gsl_test(f, "  solve_TD dim=2 A");
-  s += f;
+  {
+    double actual[] =  {0.0, 2.0};
+    f = test_TDS_solve_dim(2, 1.0, 0.5, actual, 8.0 * GSL_DBL_EPSILON);
+    gsl_test(f, "  solve_TDS dim=2 A");
+    s += f;
+  }
 
-  actual[0] =  3.0/8.0;
-  actual[1] =  15.0/8.0;
-  f = test_TD_solve_dim(2, 1.0, 1.0/3.0, actual, 8.0 * GSL_DBL_EPSILON);
-  gsl_test(f, "  solve_TD dim=2 B");
-  s += f;
+  {
+    double actual[] =  {3.0/8.0, 15.0/8.0};
+    f = test_TDS_solve_dim(2, 1.0, 1.0/3.0, actual, 8.0 * GSL_DBL_EPSILON);
+    gsl_test(f, "  solve_TDS dim=2 B");
+    s += f;
+  }
 
-  actual[0] =  5.0/8.0;
-  actual[1] =  9.0/8.0;
-  actual[2] =  2.0;
-  actual[3] =  15.0/8.0;
-  actual[4] =  35.0/8.0;
-  f = test_TD_solve_dim(5, 1.0, 1.0/3.0, actual, 8.0 * GSL_DBL_EPSILON);
-  gsl_test(f, "  solve_TD dim=5");
-  s += f;
+  {
+    double actual[] =  {5.0/8.0, 9.0/8.0, 2.0, 15.0/8.0, 35.0/8.0};
+    f = test_TDS_solve_dim(5, 1.0, 1.0/3.0, actual, 8.0 * GSL_DBL_EPSILON);
+    gsl_test(f, "  solve_TDS dim=5");
+    s += f;
+  }
 
   return s;
 }
 
 int
-test_TD_cyc_solve_one(const size_t dim, const double * d, const double * od,
+test_TDS_cyc_solve_one(const size_t dim, const double * d, const double * od,
                       const double * r, const double * actual, double eps)
 {
   int s = 0;
@@ -1755,32 +2207,62 @@ test_TD_cyc_solve_one(const size_t dim, const double * d, const double * od,
   return s;
 }
 
-int test_TD_cyc_solve(void)
+int test_TDS_cyc_solve(void)
 {
   int f;
   int s = 0;
 
-  double diag_1[] = { 1, 1, 1 };
-  double offdiag_1[] = { 3, 3, 3 };
-  double rhs_1[] = { 7, -7, 7 };
-  double actual_1[] = { -2, 5, -2 };
-  size_t dim_1 = sizeof(diag_1)/sizeof(double);
+#ifdef SUPPORT_UNDERSIZE_CYC
+  {
+    size_t dim = 1;
+    double diag[] = {  2 };
+    double offdiag[] = { 3 };
+    double rhs[] = { 7 };
+    double actual[] = { 3.5 };
+    
+    f = test_TDS_cyc_solve_one(dim, diag, offdiag, rhs, actual, 28.0 * GSL_DBL_EPSILON);
+    gsl_test(f, "  solve_TDS_cyc dim=%u A", dim);
+    s += f;
+  }
 
-  double diag_2[] = { 4, 2, 1, 2, 4 };
-  double offdiag_2[] = { 1, 1, 1, 1, 1 };
-  double rhs_2[] = { 30, -24, 3, 21, -30 };
-  double actual_2[] = { 12, 3, -42, 42, -21 };
-  size_t dim_2 = sizeof(diag_2)/sizeof(double);
+  {
+    size_t dim = 2;
+    double diag[] = { 1, 2 };
+    double offdiag[] = { 3, 4 };
+    double rhs[] = { 7, -7 };
+    double actual[] = { -5, 4 };
+    
+    f = test_TDS_cyc_solve_one(dim, diag, offdiag, rhs, actual, 28.0 * GSL_DBL_EPSILON);
+    gsl_test(f, "  solve_TDS_cyc dim=%u A", dim);
+    s += f;
+  }
+#endif
 
-  f = test_TD_cyc_solve_one(dim_1, diag_1, offdiag_1, rhs_1, actual_1, 28.0 * GSL_DBL_EPSILON);
-  gsl_test(f, "  solve_TD_cyc dim=%u A", dim_1);
-  s += f;
+  {
+    size_t dim = 3;
+    double diag[] = { 1, 1, 1 };
+    double offdiag[] = { 3, 3, 3 };
+    double rhs[] = { 7, -7, 7 };
+    double actual[] = { -2, 5, -2 };
+    
+    f = test_TDS_cyc_solve_one(dim, diag, offdiag, rhs, actual, 28.0 * GSL_DBL_EPSILON);
+    gsl_test(f, "  solve_TDS_cyc dim=%u A", dim);
+    s += f;
+  }
 
-/*  f = test_TD_cyc_solve_one(dim_2, diag_2, offdiag_2, rhs_2, actual_2, 7.0 * GSL_DBL_EPSILON);
-  FIXME: bad accuracy */
-  f = test_TD_cyc_solve_one(dim_2, diag_2, offdiag_2, rhs_2, actual_2, 35.0 * GSL_DBL_EPSILON);
-  gsl_test(f, "  solve_TD_cyc dim=%u B", dim_2);
-  s += f;
+  {
+    size_t dim = 5;
+    double diag[] = { 4, 2, 1, 2, 4 };
+    double offdiag[] = { 1, 1, 1, 1, 1 };
+    double rhs[] = { 30, -24, 3, 21, -30 };
+    double actual[] = { 12, 3, -42, 42, -21 };
+
+    /*  f = test_TDS_cyc_solve_one(dim, diag, offdiag, rhs, actual, 7.0 * GSL_DBL_EPSILON);
+        FIXME: bad accuracy */
+    f = test_TDS_cyc_solve_one(dim, diag, offdiag, rhs, actual, 35.0 * GSL_DBL_EPSILON);
+    gsl_test(f, "  solve_TDS_cyc dim=%u B", dim);
+    s += f;
+  }
 
   return s;
 }
@@ -1834,15 +2316,17 @@ int test_TDN_solve(void)
   int s = 0;
   double actual[16];
 
-  actual[0] =  3.0;
-  actual[1] = -1.0;
-  f = test_TDN_solve_dim(2, 1.0, 2.0, 1.0, actual, 1.0 * GSL_DBL_EPSILON);
+  actual[0] =  -7.0/3.0;
+  actual[1] =  5.0/3.0;
+  actual[2] =  4.0/3.0;
+  f = test_TDN_solve_dim(3, 1.0, 2.0, 1.0, actual, 2.0 * GSL_DBL_EPSILON);
   gsl_test(f, "  solve_TDN dim=2 A");
   s += f;
 
-  actual[0] =  2.0/5.0;
-  actual[1] =  9.0/5.0;
-  f = test_TDN_solve_dim(2, 1.0, 1.0/3.0, 1.0/2.0, actual, 1.0 * GSL_DBL_EPSILON);
+  actual[0] =  0.75;
+  actual[1] =  0.75;
+  actual[2] =  2.625;
+  f = test_TDN_solve_dim(3, 1.0, 1.0/3.0, 1.0/2.0, actual, 2.0 * GSL_DBL_EPSILON);
   gsl_test(f, "  solve_TDN dim=2 B");
   s += f;
 
@@ -1910,7 +2394,7 @@ int test_TDN_cyc_solve(void)
   actual[0] =  3.0/2.0;
   actual[1] = -1.0/2.0;
   actual[2] =  1.0/2.0;
-  f = test_TDN_cyc_solve_dim(3, 1.0, 2.0, 1.0, actual, 4.0 * GSL_DBL_EPSILON);
+  f = test_TDN_cyc_solve_dim(3, 1.0, 2.0, 1.0, actual, 32.0 * GSL_DBL_EPSILON);
   gsl_test(f, "  solve_TDN_cyc dim=2 A");
   s += f;
 
@@ -2035,6 +2519,9 @@ int main(void)
 {
   gsl_ieee_env_setup ();
 
+  m11 = create_general_matrix(1,1);
+  m51 = create_general_matrix(5,1);
+
   m35 = create_general_matrix(3,5);
   m53 = create_general_matrix(5,3);
   m97 = create_general_matrix(9,7);
@@ -2064,6 +2551,13 @@ int main(void)
   A33 = gsl_matrix_alloc(3,3);
   A44 = gsl_matrix_alloc(4,4);
 
+  inf5 = create_diagonal_matrix (inf5_data, 5);
+  gsl_matrix_set(inf5, 3, 3, GSL_POSINF);
+
+  nan5 = create_diagonal_matrix (inf5_data, 5);
+  gsl_matrix_set(nan5, 3, 3, GSL_NAN);
+
+
   /* Matmult now obsolete */
 #ifdef MATMULT
   gsl_test(test_matmult(),        "Matrix Multiply"); 
@@ -2074,22 +2568,28 @@ int main(void)
   gsl_test(test_LUc_solve(),      "Complex LU Decomposition and Solve");
   gsl_test(test_QR_decomp(),      "QR Decomposition");
   gsl_test(test_QR_solve(),       "QR Solve");
+  gsl_test(test_QR_QRsolve(),     "QR QR Solve");
   gsl_test(test_QR_lssolve(),     "QR LS Solve");
   gsl_test(test_QR_update(),      "QR Rank-1 Update");
   gsl_test(test_QRPT_decomp(),    "QRPT Decomposition");
   gsl_test(test_QRPT_solve(),     "QRPT Solve");
+  gsl_test(test_QRPT_QRsolve(),   "QRPT QR Solve");
   gsl_test(test_SV_decomp(),      "Singular Value Decomposition");
+  gsl_test(test_SV_decomp_mod(),  "Singular Value Decomposition (Mod)");
   gsl_test(test_SV_solve(),       "SVD Solve");
   gsl_test(test_cholesky_decomp(),"Cholesky Decomposition");
   gsl_test(test_cholesky_solve(), "Cholesky Solve");
   gsl_test(test_HH_solve(),       "Householder solve");
-  gsl_test(test_TD_solve(),       "Tridiagonal symmetric solve");
-  gsl_test(test_TD_cyc_solve(),   "Tridiagonal symmetric cyclic solve");
-
-#ifdef NONSYM
+  gsl_test(test_TDS_solve(),      "Tridiagonal symmetric solve");
+  gsl_test(test_TDS_cyc_solve(),  "Tridiagonal symmetric cyclic solve");
   gsl_test(test_TDN_solve(),      "Tridiagonal nonsymmetric solve");
   gsl_test(test_TDN_cyc_solve(),  "Tridiagonal nonsymmetric cyclic solve");
-#endif
+
+  gsl_matrix_free(m35);
+  gsl_matrix_free(m53);
+  gsl_matrix_free(m97);
+  gsl_matrix_free(s35);
+  gsl_matrix_free(s53);
 
   gsl_matrix_free(hilb2);
   gsl_matrix_free(hilb3);
@@ -2101,7 +2601,16 @@ int main(void)
   gsl_matrix_free(vander4);
   gsl_matrix_free(vander12);
 
+  gsl_matrix_free(moler10);
+
   gsl_matrix_complex_free(c7);
+  gsl_matrix_free(row3);
+  gsl_matrix_free(row5);
+  gsl_matrix_free(row12);
+
+  gsl_matrix_free(A22);
+  gsl_matrix_free(A33);
+  gsl_matrix_free(A44);
 
   exit (gsl_test_summary());
 }
