@@ -225,7 +225,7 @@ gsl_ran_discrete_preproc(size_t Kevents, const double *ProbArray)
       /* Could probably treat Kevents=1 as a special case */
 
       GSL_ERROR_VAL ("number of events must be a positive integer", 
-			GSL_EINVAL, 0);
+                        GSL_EINVAL, 0);
     }
 
     /* Make sure elements of ProbArray[] are positive.
@@ -234,8 +234,8 @@ gsl_ran_discrete_preproc(size_t Kevents, const double *ProbArray)
 
     for (k=0; k<Kevents; ++k) {
         if (ProbArray[k] < 0) {
-	  GSL_ERROR_VAL ("probabilities must be non-negative",
-			    GSL_EINVAL, 0) ;
+          GSL_ERROR_VAL ("probabilities must be non-negative",
+                            GSL_EINVAL, 0) ;
         }
         pTotal += ProbArray[k];
     }
@@ -277,10 +277,9 @@ gsl_ran_discrete_preproc(size_t Kevents, const double *ProbArray)
     while (size_stack(Smalls) > 0) {
         s = pop_stack(Smalls);
         if (size_stack(Bigs) == 0) {
-            /* Then we are on our last value */
             (g->A)[s]=s;
             (g->F)[s]=1.0;
-            break;
+            continue;
         }
         b = pop_stack(Bigs);
         (g->A)[s]=b;
@@ -310,6 +309,10 @@ gsl_ran_discrete_preproc(size_t Kevents, const double *ProbArray)
     }
     /* Stacks have been emptied, and A and F have been filled */
 
+    if ( size_stack(Smalls) != 0) {
+      GSL_ERROR_VAL ("Smalls stack has not been emptied",
+                     GSL_ESANITY, 0 );
+    }
     
 #if 0
     /* if 1, then artificially set all F[k]'s to unity.  This will
