@@ -117,45 +117,45 @@ gsl_ran_landau_pdf(const double x)
     {
       U = exp(V + 1.0);
       DENLAN = 0.3989422803 * (exp( -1 / U) / sqrt(U)) *
-	(1 + (A1[0] + (A1[1] + A1[2] * U) * U) * U);
+        (1 + (A1[0] + (A1[1] + A1[2] * U) * U) * U);
     }
   else if (V < -1)
     {
       U = exp( -V - 1);
       DENLAN = exp( -U) * sqrt(U) *
-	(P1[0] + (P1[1] + (P1[2] + (P1[3] + P1[4] * V) * V) * V) * V) /
-	(Q1[0] + (Q1[1] + (Q1[2] + (Q1[3] + Q1[4] * V) * V) * V) * V);
+        (P1[0] + (P1[1] + (P1[2] + (P1[3] + P1[4] * V) * V) * V) * V) /
+        (Q1[0] + (Q1[1] + (Q1[2] + (Q1[3] + Q1[4] * V) * V) * V) * V);
     }
   else if (V < 1)
     {
       DENLAN = (P2[0] + (P2[1] + (P2[2] + (P2[3] + P2[4] * V) * V) * V) * V) /
-	(Q2[0] + (Q2[1] + (Q2[2] + (Q2[3] + Q2[4] * V) * V) * V) * V);
+        (Q2[0] + (Q2[1] + (Q2[2] + (Q2[3] + Q2[4] * V) * V) * V) * V);
     }
   else if (V < 5)
     {
       DENLAN = (P3[0] + (P3[1] + (P3[2] + (P3[3] + P3[4] * V) * V) * V) * V) /
-	(Q3[0] + (Q3[1] + (Q3[2] + (Q3[3] + Q3[4] * V) * V) * V) * V);
+        (Q3[0] + (Q3[1] + (Q3[2] + (Q3[3] + Q3[4] * V) * V) * V) * V);
     }
   else if (V < 12)
     {
       U = 1 / V;
       DENLAN = U * U *
-	(P4[0] + (P4[1] + (P4[2] + (P4[3] + P4[4] * U) * U) * U) * U) /
-	(Q4[0] + (Q4[1] + (Q4[2] + (Q4[3] + Q4[4] * U) * U) * U) * U);
+        (P4[0] + (P4[1] + (P4[2] + (P4[3] + P4[4] * U) * U) * U) * U) /
+        (Q4[0] + (Q4[1] + (Q4[2] + (Q4[3] + Q4[4] * U) * U) * U) * U);
     }
   else if (V < 50)
     {
       U = 1 / V;
       DENLAN = U * U *
-	(P5[0] + (P5[1] + (P5[2] + (P5[3] + P5[4] * U) * U) * U) * U) /
-	(Q5[0] + (Q5[1] + (Q5[2] + (Q5[3] + Q5[4] * U) * U) * U) * U);
+        (P5[0] + (P5[1] + (P5[2] + (P5[3] + P5[4] * U) * U) * U) * U) /
+        (Q5[0] + (Q5[1] + (Q5[2] + (Q5[3] + Q5[4] * U) * U) * U) * U);
     }
   else if (V < 300)
     {
       U = 1 / V;
       DENLAN = U * U *
-	(P6[0] + (P6[1] + (P6[2] + (P6[3] + P6[4] * U) * U) * U) * U) /
-	(Q6[0] + (Q6[1] + (Q6[2] + (Q6[3] + Q6[4] * U) * U) * U) * U);
+        (P6[0] + (P6[1] + (P6[2] + (P6[3] + P6[4] * U) * U) * U) * U) /
+        (Q6[0] + (Q6[1] + (Q6[2] + (Q6[3] + Q6[4] * U) * U) * U) * U);
     }
   else
     {
@@ -316,8 +316,10 @@ gsl_ran_landau_dislan(const double x)
 double
 gsl_ran_landau(const gsl_rng * r)
 {
-  static double F[982] =
+  static double F[983] =
     {
+      0.0000000,   /* Add empty element [0] to account for difference 
+                      between C and Fortran convention for lower bound. */
       00.000000, 00.000000, 00.000000, 00.000000, 00.000000,
       -2.244733, -2.204365, -2.168163, -2.135219, -2.104898,
       -2.076740, -2.050397, -2.025605, -2.002150, -1.979866,
@@ -517,16 +519,12 @@ gsl_ran_landau(const gsl_rng * r)
       56.123356, 59.103894
     };
   double X, U, V, RANLAN;
-  int I, i;
+  int I;
 
   X = gsl_rng_uniform_pos(r);
   U = 1000.0 * X;
-  i = U;
-  U = U - i;
-
-  /* Account for difference between C and Fortran convention for lower
-     bound. */
-  I = i - 1;
+  I = U;
+  U = U - I;
 
   if (I >= 70 && I <= 800)
     {
@@ -534,8 +532,9 @@ gsl_ran_landau(const gsl_rng * r)
     }
   else if (I >= 7 && I <= 980)
     {
-      RANLAN = F[I] + U * (F[I + 1] - F[I] -
-                           0.25 * (1 - U) * (F[I + 2] - F[I + 1] - F[I] + F[I - 1]));
+      RANLAN = F[I] 
+        + U * (F[I + 1] - F[I] 
+               - 0.25 * (1 - U) * (F[I + 2] - F[I + 1] - F[I] + F[I - 1]));
     }
   else if (I < 7)
     {
