@@ -78,7 +78,10 @@ int gsl_sf_log_1plusx_mx_e(const double x, gsl_sf_result * result);
 double gsl_sf_log_1plusx_mx(const double x);
 
 
-#if HAVE_INLINE
+#ifdef HAVE_INLINE
+#include <gsl/gsl_math.h>
+#include <gsl/gsl_errno.h>
+
 extern inline
 int
 gsl_sf_log_e(const double x, gsl_sf_result * result)
