@@ -71,7 +71,7 @@ gsl_finite (const double x)
 {
   return _finite(x);
 }
-#elif defined(HAVE_IEEE_COMPARISONS)
+#elif HAVE_IEEE_COMPARISONS
 int
 gsl_isnan (const double x)
 {
@@ -100,5 +100,43 @@ gsl_finite (const double x)
   int status = (y == y);
   return status;
 }
+#else
+
+#if HAVE_DECL_ISNAN
+int
+gsl_isnan (const double x)
+{
+  return isnan(x);
+}
+#endif
+
+#if HAVE_DECL_ISINF
+int
+gsl_isinf (const double x)
+{
+    return isinf(x);
+}
+#else
+gsl_isinf (const double x)
+{
+    return (! gsl_finite(x)) && (! gsl_isnan(x));
+}
+#endif
+
+
+#if HAVE_DECL_FINITE
+int
+gsl_finite (const double x)
+{
+  return finite(x);
+}
+#elif HAVE_DECL_ISFINITE
+int
+gsl_finite (const double x)
+{
+  return isfinite(x);
+}
+#endif
+
 #endif
 
